---------------------------------------------------------------------------------
-- Einbindung Bibliotheken ------------------------------------------------------
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
---------------------------------------------------------------------------------
-- Entity -----------------------------------------------------------------------
---------------------------------------------------------------------------------
entity RS232_Interface_TB is
end RS232_Interface_TB;
---------------------------------------------------------------------------------
-- Architecture -----------------------------------------------------------------
---------------------------------------------------------------------------------
architecture Behavioral of RS232_Interface_TB is
	-- zu testende Komponente ----------------------------------------------------
	------------------------------------------------------------------------------
	component RS232_Interface
	port(	clk								: in std_logic;										-- Systemtakt
			rs232_rxd						: in std_logic;										-- RS232 RxD
			rs232_tra_en					: in std_logic;										-- Byte soll auf TxD abgesetzt werden
			rs232_dat_in					: in std_logic_vector(7 downto 0);				-- auf TxD abzusetzendes Byte
			rs232_txd						: out std_logic;										-- RS232 TxD
			rs232_rec_en					: out std_logic;										-- Byte wurde ber RxD empfangen
			rs232_txd_busy					: out std_logic;										-- RS232 TxD besetzt
			rs232_dat_out					: out std_logic_vector(7 downto 0)				-- empfangenes Byte
			);
	end component;
	------------------------------------------------------------------------------
	-- Deklarationen -------------------------------------------------------------
	------------------------------------------------------------------------------
	signal clk								: std_logic;
	signal rs232_rxd						: std_logic := '1';
	signal rs232_tra_en					: std_logic := '0';
	signal rs232_dat_in					: std_logic_vector(7 downto 0) := (others => '0');
	signal rs232_txd						: std_logic;
	signal rs232_rec_en					: std_logic;
	signal rs232_txd_busy				: std_logic;
	signal rs232_dat_out					: std_logic_vector(7 downto 0);
	------------------------------------------------------------------------------
	constant clk_period					: time := 10 ns;										-- Taktperiode bei 100 MHz
	constant baud_time					: time := 17361 ns;									-- Dauer eines Bits bei 57600 baud
begin
	-- Instanzierung -------------------------------------------------------------
	------------------------------------------------------------------------------
	UUT : RS232_Interface
	port map(clk => clk,
				rs232_rxd => rs232_rxd,
				rs232_tra_en => rs232_tra_en,
				rs232_dat_in => rs232_dat_in,
				rs232_txd => rs232_txd,
				rs232_rec_en => rs232_rec_en,
				rs232_txd_busy => rs232_txd_busy,
				rs232_dat_out => rs232_dat_out
				);
	------------------------------------------------------------------------------
	-- Systemtakt ----------------------------------------------------------------
	------------------------------------------------------------------------------
	Clock : process
	begin
		Clock_Loop : loop
			clk <= '0';
			wait for clk_period / 2;
			clk <= '1';
			wait for clk_period / 2;
		end loop Clock_Loop;
	end process;
	------------------------------------------------------------------------------
	-- Ablauf Test Bench ---------------------------------------------------------
	------------------------------------------------------------------------------
	Test_Bench : process
		procedure receive_byte
			(constant rec_byte		: std_logic_vector(7 downto 0)) is
		begin
			rs232_rxd <= '0';
			wait for baud_time;
			rs232_rxd <= rec_byte(0);
			wait for baud_time;
			rs232_rxd <= rec_byte(1);
			wait for baud_time;
			rs232_rxd <= rec_byte(2);
			wait for baud_time;
			rs232_rxd <= rec_byte(3);
			wait for baud_time;
			rs232_rxd <= rec_byte(4);
			wait for baud_time;
			rs232_rxd <= rec_byte(5);
			wait for baud_time;
			rs232_rxd <= rec_byte(6);
			wait for baud_time;
			rs232_rxd <= rec_byte(7);
			wait for baud_time;
			rs232_rxd <= '1';
			wait for baud_time;
		end receive_byte;
		---------------------------------------------------------------------------
		procedure transmit_byte
			(constant tra_byte		: std_logic_vector(7 downto 0)) is
		begin
			rs232_dat_in <= tra_byte;
			wait for 10 ns;
			rs232_tra_en <= '1';
			wait for 10 ns;
			rs232_tra_en <= '0';
		end transmit_byte;
		---------------------------------------------------------------------------
	begin
		--rs232_tra_en <= '1';
		wait for 10000 ns;
		transmit_byte(X"BB");
		wait for 10000 ns;
		receive_byte(X"AA");
		wait for 10000 ns;
		--receive_byte(X"BB");
		--wait for 10000 ns;
		--transmit_byte(X"AA");
		--wait for 200000 ns;
		--transmit_byte(X"BB");
		--wait;
	end process;
	------------------------------------------------------------------------------
end Behavioral;
