----------------------------------------------------------------------------------
-- Company: 			no company, hobby engineering
-- Engineer: 			befro
-- 
-- Create Date:    	10:10:27 03/19/2009 
-- Design Name: 		
-- Module Name:    	RamInterface - Behavioral 
-- Project Name: 		PollinCPLD-Board Logic Analysator
-- Target Devices: 
-- Tool versions: 
-- Description: 		Interface fr das auf dem PollinCPLD-Board befindliche 
--							128Kx8 SRAM
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity RamInterface is
    Port ( 
				-- RAM SIGNALS
				ramAddress : out  STD_LOGIC_VECTOR (16 downto 0);
				ramData : inout  STD_LOGIC_VECTOR (7 downto 0):="ZZZZZZZZ";
				ramNWE : out  STD_LOGIC;
				ramNOE : out  STD_LOGIC;
			  
				-- Interface Signals
				dataOut : out  STD_LOGIC_VECTOR (7 downto 0);
				dataIn : in  STD_LOGIC_VECTOR (7 downto 0);
				clk : in  STD_LOGIC;
				clkNextValue : in  STD_LOGIC;
				ReadOrNSample : in  STD_LOGIC;
				Reset : in STD_LOGIC);	
end RamInterface;

architecture Behavioral of RamInterface is
signal ramAddressCounter: STD_LOGIC_VECTOR (16 downto 0 ):="00000000000000000";
signal rns_delayed: STD_LOGIC:='1';
signal ramDataOutEnable: STD_LOGIC:='0'; --disable cpld output buffer

begin
EinProzessModell: process(clk,clkNextValue,readOrNSample,ramAddressCounter, dataIn)
begin
	
	if rising_edge(clkNextValue) then
		if Reset = '0' then
			ramAddressCounter <= ramAddressCounter+1;
		else
			ramAddressCounter <= ( others => '0' );
		end if;	
		rns_delayed <= readOrNSample; -- verzgertes read/write Umschaltsignal
	end if;	
	
	if readOrNSample = '1' then -- Daten vom Ram lesen
		ramNWE <= '1'; -- Ram write disable
	else -- Daten ins Ram schreiben
		ramNWE <= clkNextValue or clk;
	end if;
end process EinProzessModell;

ramData <= 	dataIn when ramDataOutEnable = '1' else -- Daten ins Ram schreiben
					"ZZZZZZZZ" when ramDataOutEnable = '0'; --hochohmig, damit Daten vom Ram gelesen werden knnen
dataOut <= ramData; --Daten vom Ram lesen

ramDataOutEnable <= not (rns_delayed or readOrNSample );
ramNOE <= not (rns_delayed and readOrNSample );

ramAddress <= ramAddressCounter;

end Behavioral;

