----------------------------------------------------------------------------------
-- Company: 			no company, hobby engineering
-- Engineer: 		 	befro
-- 
-- Create Date:    	08:18:34 03/18/2009 
-- Design Name: 
-- Module Name:    	stateMachine - Behavioral 
-- Project Name: 		PollinCPLD-Board Logic Analysator
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity stateMachine is
    Port ( RxData : in  STD_LOGIC_VECTOR (7 downto 0); -- Daten von der seriellen Schnittstelle
           TxData : out  STD_LOGIC_VECTOR (7 downto 0); 
           TxPulse : out  STD_LOGIC; -- Signalisierung Sendedaten verfgbar
           statusIndicator : out  STD_LOGIC;
			  readRam: out STD_LOGIC; -- auslesen des Rams
			  ramAddressCLK: out STD_LOGIC:='0'; -- ClK fr RAM-Adresse
			  clk : in STD_LOGIC ); -- Systemtakt
			
end stateMachine;

architecture Behavioral of stateMachine is

type zustaende is ( START, a_SENDWERT_Z1, a_SENDWERT_Z2,d_SAMPLE_Z1);
signal zustand, folgezustand: zustaende;
signal counter: STD_LOGIC_VECTOR( 7 downto 0 ):= ( others => '0');
signal txflag: STD_LOGIC := '0'; -- default: keine Daten verfgbar
signal sampleFlag: STD_LOGIC := '0'; -- default: Ram lesen, nicht sampeln
signal divider: STD_LOGIC_VECTOR (2 downto 0 ):= "000";

begin
	z_speicher: process(clk)
	begin 
		
		if rising_edge(clk) then
			txflag <='0';
			counter <= counter ;
			
			if zustand = folgezustand then
				else 
					txflag <= '1';
					counter <= counter+1;
			end if;

			if zustand = d_SAMPLE_Z1 then
				sampleFlag <= '1';
			else
				sampleFlag <= '0';
			end if;
				
			zustand <= folgezustand;
			divider<=divider+1;

		end if;
			
	end process z_speicher;	

	ue_sn: process( RxData, zustand, counter )
	begin
		case zustand is
			-- zustand: transfer byte to pc
			when START				=> 	folgezustand <= START;
												if RxData = "01100001" then -- 'a'=0x61
													folgezustand <= a_SENDWERT_Z1;
													elsif RxData = "01100100" then -- 'd'=0x64
														folgezustand <= d_SAMPLE_Z1;
													end if;				
			when a_SENDWERT_Z1 	=> 	folgezustand <= a_SENDWERT_Z2;
											
			-- zustand: transfer byte to pc
			when a_SENDWERT_Z2	=> 	if RxData = "01110011" then -- 's'=0x73; zurck in den Startzustand
													folgezustand <= START;
												else 	folgezustand <= a_SENDWERT_Z2; -- im Zustand z2 bleiben
												end if;
			-- zustand: full speed sampling
			when d_SAMPLE_Z1		=>		if RxData = "01110011" then -- 's'=0x73; zurck in den Startzustand
													folgezustand <= START;
												else 	folgezustand <= d_SAMPLE_Z1; -- im Zustand d_SAMPLE_Z1 bleiben
												end if;
			when others				=>		folgezustand <= zustand; -- zustand nicht verndern
											
		end case;
	end process ue_sn;
	
	ramAddressCLK <= divider(2) when sampleFlag='1'	else txflag;
			
	TxData <= RxData;
	TxPulse <= txflag;

	statusIndicator <= sampleFlag;
	readRam <= not sampleFlag;
	
end Behavioral;

